@@ -1,20 +1,21 @@
 
 --- Module implementing the luarocks-admin "add" command.
 -- Adds a rock or rockspec to a rocks server.
-module("luarocks.add", package.seeall)
+--module("luarocks.add", package.seeall)
+local add = {}
+package.loaded["luarocks.add"] = add
 
 local cfg = require("luarocks.cfg")
 local util = require("luarocks.util")
-local fetch = require("luarocks.fetch")
 local dir = require("luarocks.dir")
 local manif = require("luarocks.manif")
 local index = require("luarocks.index")
 local fs = require("luarocks.fs")
 local cache = require("luarocks.cache")
 
-help_summary = "Add a rock or rockspec to a rocks server."
-help_arguments = "[--server=<server>] [--no-refresh] {<rockspec>|<rock>...}"
-help = [[
+add.help_summary = "Add a rock or rockspec to a rocks server."
+add.help_arguments = "[--server=<server>] [--no-refresh] {<rockspec>|<rock>...}"
+add.help = [[
 Arguments are local files, which may be rockspecs or rocks.
 The flag --server indicates which server to use.
 If not given, the default server set in the upload_server variable
@@ -45,7 +46,8 @@ local function add_files_to_server(refresh, rockfiles, server, upload_server)
       login_url = protocol.."://"..server_path
    end
    
-   fs.change_dir(at)
+   local ok, err = fs.change_dir(at)
+   if not ok then return nil, err end
    
    local files = {}
    for i, rockfile in ipairs(rockfiles) do
@@ -62,10 +64,14 @@ local function add_files_to_server(refresh, rockfiles, server, upload_server)
       return nil, "No files found"
    end
 
-   fs.change_dir(local_cache)
+   local ok, err = fs.change_dir(local_cache)
+   if not ok then return nil, err end
 
    util.printout("Updating manifest...")
-   manif.make_manifest(local_cache, "one")
+   manif.make_manifest(local_cache, "one", true)
+   
+   manif.zip_manifests()
+   
    util.printout("Updating index.html...")
    index.make_index(local_cache)
 
@@ -76,17 +82,24 @@ local function add_files_to_server(refresh, rockfiles, server, upload_server)
       login_url = login_url .. "/"
    end
 
+   table.insert(files, "index.html")
+   table.insert(files, "manifest")
+   for ver in util.lua_versions() do
+      table.insert(files, "manifest-"..ver)
+      table.insert(files, "manifest-"..ver..".zip")
+   end
+
    -- TODO abstract away explicit 'curl' call
 
    local cmd
    if protocol == "rsync" then
       local srv, path = server_path:match("([^/]+)(/.+)")
-      cmd = cfg.variables.RSYNC.." --exclude=.git -Oavz -e ssh "..local_cache.."/ "..user.."@"..srv..":"..path.."/"
+      cmd = cfg.variables.RSYNC.." "..cfg.variables.RSYNCFLAGS.." -e ssh "..local_cache.."/ "..user.."@"..srv..":"..path.."/"
    elseif upload_server and upload_server.sftp then
       local part1, part2 = upload_server.sftp:match("^([^/]*)/(.*)$")
-      cmd = cfg.variables.SCP.." manifest index.html "..table.concat(files, " ").." "..user.."@"..part1..":/"..part2
+      cmd = cfg.variables.SCP.." "..table.concat(files, " ").." "..user.."@"..part1..":/"..part2
    else
-      cmd = cfg.variables.CURL.." "..login_info.." -T '{manifest,index.html,"..table.concat(files, ",").."}' "..login_url
+      cmd = cfg.variables.CURL.." "..login_info.." -T '{"..table.concat(files, ",").."}' "..login_url
    end
 
    util.printout(cmd)
@@ -95,14 +108,16 @@ local function add_files_to_server(refresh, rockfiles, server, upload_server)
    return true
 end
 
-function run(...)
+function add.run(...)
    local files = { util.parse_flags(...) }
    local flags = table.remove(files, 1)
    if #files < 1 then
-      return nil, "Argument missing, see help."
+      return nil, "Argument missing. "..util.see_help("add", "luarocks-admin")
    end
    local server, server_table = cache.get_upload_server(flags["server"])
    if not server then return nil, server_table end
    return add_files_to_server(not flags["no-refresh"], files, server, server_table)
 end
 
+
+return add
