@@ -1,7 +1,9 @@
 
 --- Module implementing the LuaRocks "build" command.
 -- Builds a rock, compiling its C parts if any.
-module("luarocks.build", package.seeall)
+--module("luarocks.build", package.seeall)
+local build = {}
+package.loaded["luarocks.build"] = build
 
 local pack = require("luarocks.pack")
 local path = require("luarocks.path")
@@ -12,19 +14,30 @@ local fs = require("luarocks.fs")
 local dir = require("luarocks.dir")
 local deps = require("luarocks.deps")
 local manif = require("luarocks.manif")
+local remove = require("luarocks.remove")
 local cfg = require("luarocks.cfg")
 
-help_summary = "Build/compile a rock."
-help_arguments = "[--pack-binary-rock] {<rockspec>|<rock>|<name> [<version>]}"
-help = [[
+build.help_summary = "Build/compile a rock."
+build.help_arguments = "[--pack-binary-rock] [--keep] {<rockspec>|<rock>|<name> [<version>]}"
+build.help = [[
 Build and install a rock, compiling its C parts if any.
 Argument may be a rockspec file, a source rock file
 or the name of a rock to be fetched from a repository.
 
-If --pack-binary-rock is passed, the rock is not installed;
-instead, a .rock file with the contents of compilation is produced
-in the current directory.
-]]
+--pack-binary-rock  Do not install rock. Instead, produce a .rock file
+                    with the contents of compilation in the current
+                    directory.
+
+--keep              Do not remove previously installed versions of the
+                    rock after building a new one. This behavior can
+                    be made permanent by setting keep_other_versions=true
+                    in the configuration file.
+
+--branch=<name>     Override the `source.branch` field in the loaded
+                    rockspec. Allows to specify a different branch to 
+                    fetch. Particularly for SCM rocks.
+
+]]..util.deps_mode_help()
 
 --- Install files to a given location.
 -- Takes a table where the array part is a list of filenames to be copied.
@@ -46,19 +59,31 @@ local function install_files(files, location, is_module_path)
    if files then
       for k, file in pairs(files) do
          local dest = location
+         local filename = dir.base_name(file)
          if type(k) == "string" then
+            local modname = k
             if is_module_path then
-               dest = dir.path(location, path.module_to_path(k))
-               fs.make_dir(dest)
+               dest = dir.path(location, path.module_to_path(modname))
+               local ok, err = fs.make_dir(dest)
+               if not ok then return nil, err end
+               if filename:match("%.lua$") then
+                  local basename = modname:match("([^.]+)$")
+                  local baseinfo = filename:gsub("%.lua$", "")
+                  if basename ~= baseinfo then
+                     filename = basename..".lua"
+                  end
+               end
             else
-               dest = dir.path(location, dir.dir_name(k))
-               fs.make_dir(dest)
-               dest = dir.path(dest, dir.base_name(k))
+               dest = dir.path(location, dir.dir_name(modname))
+               local ok, err = fs.make_dir(dest)
+               if not ok then return nil, err end
+               filename = dir.base_name(modname)
             end
          else
-            fs.make_dir(dest)
+            local ok, err = fs.make_dir(dest)
+            if not ok then return nil, err end
          end
-         local ok = fs.copy(dir.path(file), dest)
+         local ok = fs.copy(dir.path(file), dir.path(dest, filename))
          if not ok then
             return nil, "Failed copying "..file
          end
@@ -84,16 +109,16 @@ end
 -- @param rockspec table: A rockspec table.
 -- @return boolean or (nil, string): True if succeeded or 
 -- nil and an error message.
-function apply_patches(rockspec)
+function build.apply_patches(rockspec)
    assert(type(rockspec) == "table")
 
-   local build = rockspec.build
-   if build.extra_files then
-      extract_from_rockspec(build.extra_files)
+   local build_spec = rockspec.build
+   if build_spec.extra_files then
+      extract_from_rockspec(build_spec.extra_files)
    end
-   if build.patches then
-      extract_from_rockspec(build.patches)
-      for patch, patchdata in util.sortedpairs(build.patches) do
+   if build_spec.patches then
+      extract_from_rockspec(build_spec.patches)
+      for patch, patchdata in util.sortedpairs(build_spec.patches) do
          util.printout("Applying patch "..patch.."...")
          local ok, err = fs.apply_patch(tostring(patch), patchdata)
          if not ok then
@@ -104,6 +129,24 @@ function apply_patches(rockspec)
    return true
 end
 
+local function install_default_docs(name, version)
+   local patterns = { "readme", "license", "copying" }
+   local dest = dir.path(path.install_dir(name, version), "doc")
+   local has_dir = false
+   for name in fs.dir() do
+      for _, pattern in ipairs(patterns) do
+         if name:lower():match("^"..pattern) then
+            if not has_dir then
+               fs.make_dir(dest)
+               has_dir = true
+            end
+            fs.copy(name, dest)
+            break
+         end
+      end
+   end
+end
+
 --- Build and install a rock given a rockspec.
 -- @param rockspec_file string: local or remote filename of a rockspec.
 -- @param need_to_fetch boolean: true if sources need to be fetched,
@@ -111,11 +154,12 @@ end
 -- @param minimal_mode boolean: true if there's no need to fetch,
 -- unpack or change dir (this is used by "luarocks make"). Implies
 -- need_to_fetch = false.
--- @param deps_mode: string: Which trees to check dependencies for:
--- "none", "one", "order" or "all".
--- @return boolean or (nil, string, [string]): True if succeeded or 
--- nil and an error message followed by an error code.
-function build_rockspec(rockspec_file, need_to_fetch, minimal_mode, deps_mode)
+-- @param deps_mode string: Dependency mode: "one" for the current default tree,
+-- "all" for all trees, "order" for all trees with priority >= the current default,
+-- "none" for no trees.
+-- @return (string, string) or (nil, string, [string]): Name and version of
+-- installed rock if succeeded or nil and an error message followed by an error code.
+function build.build_rockspec(rockspec_file, need_to_fetch, minimal_mode, deps_mode)
    assert(type(rockspec_file) == "string")
    assert(type(need_to_fetch) == "boolean")
 
@@ -137,7 +181,7 @@ function build_rockspec(rockspec_file, need_to_fetch, minimal_mode, deps_mode)
       end
    end
 
-   ok, err, errcode = deps.check_external_deps(rockspec, "build")
+   local ok, err, errcode = deps.check_external_deps(rockspec, "build")
    if err then
       return nil, err, errcode
    end
@@ -154,7 +198,8 @@ function build_rockspec(rockspec_file, need_to_fetch, minimal_mode, deps_mode)
          if not ok then
             return nil, source_dir, errcode
          end
-         fs.change_dir(source_dir)
+         local ok, err = fs.change_dir(source_dir)
+         if not ok then return nil, err end
       elseif rockspec.source.file then
          local ok, err = fs.unpack_archive(rockspec.source.file)
          if not ok then
@@ -172,34 +217,39 @@ function build_rockspec(rockspec_file, need_to_fetch, minimal_mode, deps_mode)
    }
    
    for _, d in pairs(dirs) do
-      fs.make_dir(d.name)
+      local ok, err = fs.make_dir(d.name)
+      if not ok then return nil, err end
    end
    local rollback = util.schedule_function(function()
       fs.delete(path.install_dir(name, version))
       fs.remove_dir_if_empty(path.versions_dir(name))
    end)
 
-   local build = rockspec.build
+   local build_spec = rockspec.build
    
    if not minimal_mode then
-      ok, err = apply_patches(rockspec)
+      ok, err = build.apply_patches(rockspec)
       if err then
          return nil, err
       end
    end
    
-   if build.type ~= "none" then
+   if build_spec.type ~= "none" then
 
       -- Temporary compatibility
-      if build.type == "module" then
+      if build_spec.type == "module" then
          util.printout("Do not use 'module' as a build type. Use 'builtin' instead.")
-         build.type = "builtin"
+         build_spec.type = "builtin"
+      end
+
+      if cfg.accepted_build_types and util.array_contains(cfg.accepted_build_types, build_spec.type) then
+         return nil, "This rockspec uses the '"..build_spec.type.."' build type, which is blocked by the 'accepted_build_types' setting in your LuaRocks configuration."
       end
 
       local build_type
-      ok, build_type = pcall(require, "luarocks.build." .. build.type)
+      ok, build_type = pcall(require, "luarocks.build." .. build_spec.type)
       if not ok or not type(build_type) == "table" then
-         return nil, "Failed initializing build back-end for build type '"..build.type.."': "..build_type
+         return nil, "Failed initializing build back-end for build type '"..build_spec.type.."': "..build_type
       end
   
       ok, err = build_type.run(rockspec)
@@ -208,27 +258,40 @@ function build_rockspec(rockspec_file, need_to_fetch, minimal_mode, deps_mode)
       end
    end
 
-   if build.install then
+   if build_spec.install then
       for id, install_dir in pairs(dirs) do
-         ok, err = install_files(build.install[id], install_dir.name, install_dir.is_module_path)
+         ok, err = install_files(build_spec.install[id], install_dir.name, install_dir.is_module_path)
          if not ok then 
             return nil, err
          end
       end
    end
    
-   local copy_directories = build.copy_directories or {"doc"}
+   local copy_directories = build_spec.copy_directories
+   local copying_default = false
+   if not copy_directories then
+      copy_directories = {"doc"}
+      copying_default = true
+   end
 
+   local any_docs = false
    for _, copy_dir in pairs(copy_directories) do
       if fs.is_dir(copy_dir) then
          local dest = dir.path(path.install_dir(name, version), copy_dir)
          fs.make_dir(dest)
          fs.copy_contents(copy_dir, dest)
+         any_docs = true
       else
-         util.warning("Directory '"..copy_dir.."' not found")
+         if not copying_default then
+            return nil, "Directory '"..copy_dir.."' not found"
+         end
       end
    end
-
+   
+   if not any_docs then
+      install_default_docs(name, version)
+   end
+   
    for _, d in pairs(dirs) do
       fs.remove_dir_if_empty(d.name)
    end
@@ -263,11 +326,11 @@ function build_rockspec(rockspec_file, need_to_fetch, minimal_mode, deps_mode)
    end
 
    local root_dir = path.root_dir(cfg.rocks_dir)
-   util.printout()
    util.printout(name.." "..version.." is now built and installed in "..root_dir.." "..license)
+   util.printout()
    
    util.remove_scheduled_function(rollback)
-   return true
+   return name, version
 end
 
 --- Build and install a rock.
@@ -275,10 +338,11 @@ end
 -- @param need_to_fetch boolean: true if sources need to be fetched,
 -- false if the rockspec was obtained from inside a source rock.
 -- @param deps_mode: string: Which trees to check dependencies for:
--- "none", "one", "order" or "all".
+-- "one" for the current default tree, "all" for all trees,
+-- "order" for all trees with priority >= the current default, "none" for no trees.
 -- @return boolean or (nil, string, [string]): True if build was successful,
 -- or false and an error message and an optional error code.
-function build_rock(rock_file, need_to_fetch, deps_mode)
+function build.build_rock(rock_file, need_to_fetch, deps_mode)
    assert(type(rock_file) == "string")
    assert(type(need_to_fetch) == "boolean")
   
@@ -287,25 +351,26 @@ function build_rock(rock_file, need_to_fetch, deps_mode)
       return nil, err, errcode
    end
    local rockspec_file = path.rockspec_name_from_rock(rock_file)
-   fs.change_dir(unpack_dir)
-   local ok, err, errcode = build_rockspec(rockspec_file, need_to_fetch, false, deps_mode)
+   local ok, err = fs.change_dir(unpack_dir)
+   if not ok then return nil, err end
+   local ok, err, errcode = build.build_rockspec(rockspec_file, need_to_fetch, false, deps_mode)
    fs.pop_dir()
    return ok, err, errcode
 end
  
 local function do_build(name, version, deps_mode)
    if name:match("%.rockspec$") then
-      return build_rockspec(name, true, false, deps_mode)
+      return build.build_rockspec(name, true, false, deps_mode)
    elseif name:match("%.src%.rock$") then
-      return build_rock(name, false, deps_mode)
+      return build.build_rock(name, false, deps_mode)
    elseif name:match("%.all%.rock$") then
       local install = require("luarocks.install")
       return install.install_binary_rock(name, deps_mode)
    elseif name:match("%.rock$") then
-      return build_rock(name, true, deps_mode)
+      return build.build_rock(name, true, deps_mode)
    elseif not name:match(dir.separator) then
       local search = require("luarocks.search")
-      return search.act_on_src_or_rockspec(run, name:lower(), version, deps.deps_mode_to_flag(deps_mode))
+      return search.act_on_src_or_rockspec(build.run, name:lower(), version, deps.deps_mode_to_flag(deps_mode))
    end
    return nil, "Don't know what to do with "..name
 end
@@ -316,12 +381,12 @@ end
 -- if returned a result, installs the matching rock.
 -- @param version string: When passing a package name, a version number may
 -- also be given.
--- @return boolean or (nil, string): True if build was successful; nil and an
--- error message otherwise.
-function run(...)
+-- @return boolean or (nil, string, exitcode): True if build was successful; nil and an
+-- error message otherwise. exitcode is optionally returned.
+function build.run(...)
    local flags, name, version = util.parse_flags(...)
    if type(name) ~= "string" then
-      return nil, "Argument missing, see help."
+      return nil, "Argument missing. "..util.see_help("build")
    end
    assert(type(version) == "string" or not version)
 
@@ -329,7 +394,16 @@ function run(...)
       return pack.pack_binary_rock(name, version, do_build, name, version, deps.get_deps_mode(flags))
    else
       local ok, err = fs.check_command_permissions(flags)
+      if not ok then return nil, err, cfg.errorcodes.PERMISSIONDENIED end
+      ok, err = do_build(name, version, deps.get_deps_mode(flags))
       if not ok then return nil, err end
-      return do_build(name, version, deps.get_deps_mode(flags))
+      local name, version = ok, err
+      if (not flags["keep"]) and not cfg.keep_other_versions then
+         local ok, err = remove.remove_other_versions(name, version, flags["force"])
+         if not ok then util.printerr(err) end
+      end
+      return name, version
    end
 end
+
+return build
