@@ -1,24 +1,42 @@
 
 --- Module implementing the luarocks "download" command.
 -- Download a rock from the repository.
-module("luarocks.download", package.seeall)
+--module("luarocks.download", package.seeall)
+local download = {}
+package.loaded["luarocks.download"] = download
 
 local util = require("luarocks.util")
 local path = require("luarocks.path")
 local fetch = require("luarocks.fetch")
 local search = require("luarocks.search")
+local fs = require("luarocks.fs")
+local dir = require("luarocks.dir")
 
-help_summary = "Download a specific rock file from a rocks server."
-help_arguments = "[--all] [--arch=<arch> | --source | --rockspec] [<name> [<version>]]"
+download.help_summary = "Download a specific rock file from a rocks server."
+download.help_arguments = "[--all] [--arch=<arch> | --source | --rockspec] [<name> [<version>]]"
 
-help = [[
+download.help = [[
 --all          Download all files if there are multiple matches.
 --source       Download .src.rock if available.
 --rockspec     Download .rockspec if available.
 --arch=<arch>  Download rock for a specific architecture.
 ]]
 
-function download(arch, name, version, all)
+local function get_file(filename)
+   local protocol, pathname = dir.split_url(filename)
+   if protocol == "file" then
+      local ok, err = fs.copy(pathname, fs.current_dir())
+      if ok then
+         return pathname
+      else
+         return nil, err
+      end
+   else
+      return fetch.fetch_url(filename)
+   end
+end
+
+function download.download(arch, name, version, all)
    local results, err
    local query = search.make_query(name, version)
    if arch then query.arch = arch end
@@ -29,8 +47,7 @@ function download(arch, name, version, all)
       results, err = search.find_suitable_rock(query)
    end
    if type(results) == "string" then
-      local file = fetch.fetch_url(results)
-      return file
+      return get_file(results)
    elseif type(results) == "table" and next(results) then
       if all then
          local all_ok = true
@@ -39,7 +56,7 @@ function download(arch, name, version, all)
             for version, versions in pairs(result) do
                for _,items in pairs(versions) do
                   local filename = path.make_url(items.repo, name, version, items.arch)
-                  local ok, err = fetch.fetch_url(filename)
+                  local ok, err = get_file(filename)
                   if not ok then
                      all_ok = false
                      any_err = any_err .. "\n" .. err
@@ -64,7 +81,7 @@ end
 -- version may also be passed.
 -- @return boolean or (nil, string): true if successful or nil followed
 -- by an error message.
-function run(...)
+function download.run(...)
    local flags, name, version = util.parse_flags(...)
    
    assert(type(version) == "string" or not version)
@@ -83,6 +100,8 @@ function run(...)
       arch = flags["arch"]
    end
    
-   local dl, err = download(arch, name, version, flags["all"])
+   local dl, err = download.download(arch, name, version, flags["all"])
    return dl and true, err
 end
+
+return download
