@@ -2,7 +2,8 @@
 --- fs operations implemented with third-party tools for Windows platform abstractions.
 -- Download http://unxutils.sourceforge.net/ for Windows GNU utilities
 -- used by this module.
-module("luarocks.fs.win32.tools", package.seeall)
+--module("luarocks.fs.win32.tools", package.seeall)
+local tools = {}
 
 local fs = require("luarocks.fs")
 local dir = require("luarocks.dir")
@@ -12,10 +13,6 @@ local dir_stack = {}
 
 local vars = cfg.variables
 
-local function pack(...)
-   return { n = select("#", ...), ... }
-end
-
 --- Strip the last extension of a filename.
 -- Example: "foo.tar.gz" becomes "foo.tar".
 -- If filename has no dots, returns it unchanged.
@@ -39,10 +36,14 @@ end
 --- Obtain current directory.
 -- Uses the module's internal directory stack.
 -- @return string: the absolute pathname of the current directory.
-function current_dir()
-   local pipe = io.popen(vars.PWD)
-   local current = pipe:read("*l")
-   pipe:close()
+function tools.current_dir()
+   local current = cfg.cache_pwd
+   if not current then
+      local pipe = io.popen(fs.Q(vars.PWD))
+      current = pipe:read("*l")
+      pipe:close()
+      cfg.cache_pwd = current
+   end
    for _, directory in ipairs(dir_stack) do
       current = fs.absolute_name(directory, current)
    end
@@ -54,18 +55,10 @@ end
 -- @param cmd string: No quoting/escaping is applied to the command.
 -- @return boolean: true if command succeeds (status code 0), false
 -- otherwise.
-function execute_string(cmd)
+function tools.execute_string(cmd)
    cmd = command_at(fs.current_dir(), cmd)
-   if cfg.verbose then print("Executing: "..tostring(cmd)) end
-   local code = pack(os.execute(cmd))
-   if cfg.verbose then
-      print("Results: "..tostring(code.n))
-      for i = 1,code.n do
-         print("  "..tostring(i).." ("..type(code[i]).."): "..tostring(code[i]))
-      end
-      print()
-   end
-   if code[1] == 0 or code[1] == true then
+   local code = os.execute(cmd)
+   if code == 0 or code == true then
       return true
    else
       return false
@@ -77,20 +70,25 @@ end
 -- semantics of chdir, as it does not handle errors the same way,
 -- but works well for our purposes for now.
 -- @param directory string: The directory to switch to.
-function change_dir(directory)
+-- @return boolean or (nil, string): true if successful, (nil, error message) if failed.
+function tools.change_dir(directory)
    assert(type(directory) == "string")
-   table.insert(dir_stack, directory)
+   if fs.is_dir(directory) then
+      table.insert(dir_stack, directory)
+      return true
+   end
+   return nil, "directory not found: "..directory
 end
 
 --- Change directory to root.
 -- Allows leaving a directory (e.g. for deleting it) in
 -- a crossplatform way.
-function change_dir_to_root()
+function tools.change_dir_to_root()
    table.insert(dir_stack, "/")
 end
 
 --- Change working directory to the previous in the directory stack.
-function pop_dir()
+function tools.pop_dir()
    local directory = table.remove(dir_stack)
    return directory ~= nil
 end
@@ -100,28 +98,32 @@ end
 -- too, they are created as well.
 -- @param directory string: pathname of directory to create.
 -- @return boolean: true on success, false on failure.
-function make_dir(directory)
+function tools.make_dir(directory)
    assert(directory)
-   fs.execute(fs.quiet(vars.MKDIR.." "..fs.Q(directory)))
-   return 1
+   directory = dir.normalize(directory)
+   fs.execute_quiet(fs.Q(vars.MKDIR).." -p ", directory)
+   if not fs.is_dir(directory) then
+      return false, "failed making directory "..directory
+   end
+   return true
 end
 
 --- Remove a directory if it is empty.
 -- Does not return errors (for example, if directory is not empty or
 -- if already does not exist)
 -- @param directory string: pathname of directory to remove.
-function remove_dir_if_empty(directory)
+function tools.remove_dir_if_empty(directory)
    assert(directory)
-   fs.execute_string(fs.quiet(vars.RMDIR.." "..fs.Q(directory)))
+   fs.execute_quiet(fs.Q(vars.RMDIR), directory)
 end
 
 --- Remove a directory if it is empty.
 -- Does not return errors (for example, if directory is not empty or
 -- if already does not exist)
 -- @param directory string: pathname of directory to remove.
-function remove_dir_tree_if_empty(directory)
+function tools.remove_dir_tree_if_empty(directory)
    assert(directory)
-   fs.execute_string(fs.quiet(vars.RMDIR.." "..fs.Q(directory)))
+   fs.execute_quiet(fs.Q(vars.RMDIR), directory)
 end
 
 --- Copy a file.
@@ -129,10 +131,11 @@ end
 -- @param dest string: Pathname of destination
 -- @return boolean or (boolean, string): true on success, false on failure,
 -- plus an error message.
-function copy(src, dest)
+function tools.copy(src, dest)
    assert(src and dest)
    if dest:match("[/\\]$") then dest = dest:sub(1, -2) end
-   if fs.execute(vars.CP, src, dest) then
+   local ok = fs.execute(fs.Q(vars.CP), src, dest)
+   if ok then
       return true
    else
       return false, "Failed copying "..src.." to "..dest
@@ -144,10 +147,9 @@ end
 -- @param dest string: Pathname of destination
 -- @return boolean or (boolean, string): true on success, false on failure,
 -- plus an error message.
-function copy_contents(src, dest)
+function tools.copy_contents(src, dest)
    assert(src and dest)
-   if fs.execute_string(fs.quiet("xcopy "..src.."\\*.* "..fs.Q(dest).." /S/E/Y")) then
-   --if fs.execute_string(fs.quiet(vars.CP.." -a "..src.."\\*.* "..fs.Q(dest))) then
+   if fs.execute_quiet(fs.Q(vars.CP).." -dR "..src.."\\*.* "..fs.Q(dest)) then
       return true
    else
       return false, "Failed copying "..src.." to "..dest
@@ -157,35 +159,25 @@ end
 --- Delete a file or a directory and all its contents.
 -- For safety, this only accepts absolute paths.
 -- @param arg string: Pathname of source
--- @return boolean: true on success, false on failure.
-function delete(arg)
+-- @return nil
+function tools.delete(arg)
    assert(arg)
-   assert(arg:match("^[\a-zA-Z]?:?[\\/]"))
-   fs.execute(vars.CHMOD.." a+rw -R ", arg)
-   return fs.execute_string(fs.quiet(vars.RM.." -rf " .. fs.Q(arg)))
+   assert(arg:match("^[a-zA-Z]?:?[\\/]"))
+   fs.execute_quiet("if exist "..fs.Q(arg.."\\").." ( RMDIR /S /Q "..fs.Q(arg).." ) else ( DEL /Q /F "..fs.Q(arg).." )")
 end
 
---- List the contents of a directory.
--- @param at string or nil: directory to list (will be the current
--- directory if none is given).
--- @return table: an array of strings with the filenames representing
--- the contents of a directory.
-function list_dir(at)
-   assert(type(at) == "string" or not at)
-   if not at then
-      at = fs.current_dir()
-   end
-   if not fs.is_dir(at) then
-      return {}
-   end
-   local result = {}
-   local pipe = io.popen(command_at(at, vars.LS))
+--- Internal implementation function for fs.dir.
+-- Yields a filename on each iteration.
+-- @param at string: directory to list
+-- @return nil
+function tools.dir_iterator(at)
+   local pipe = io.popen(command_at(at, fs.Q(vars.LS)))
    for file in pipe:lines() do
-      table.insert(result, file)
+      if file ~= "." and file ~= ".." then
+         coroutine.yield(file)
+      end
    end
    pipe:close()
-
-   return result
 end
 
 --- Recursively scan the contents of a directory.
@@ -193,7 +185,7 @@ end
 -- directory if none is given).
 -- @return table: an array of strings with the filenames representing
 -- the contents of a directory. Paths are returned with forward slashes.
-function find(at)
+function tools.find(at)
    assert(type(at) == "string" or not at)
    if not at then
       at = fs.current_dir()
@@ -202,7 +194,7 @@ function find(at)
       return {}
    end
    local result = {}
-   local pipe = io.popen(command_at(at, vars.FIND.." 2> NUL"))
+   local pipe = io.popen(command_at(at, fs.Q(vars.FIND).." 2> NUL"))
    for file in pipe:lines() do
       -- Windows find is a bit different
       local first_two = file:sub(1,2)
@@ -220,32 +212,32 @@ end
 -- @param ... Filenames to be stored in the archive are given as
 -- additional arguments.
 -- @return boolean: true on success, false on failure.
-function zip(zipfile, ...)
-   return fs.execute(vars.SEVENZ.." a -tzip", zipfile, ...)
+function tools.zip(zipfile, ...)
+   return fs.execute_quiet(fs.Q(vars.SEVENZ).." -aoa a -tzip", zipfile, ...)
 end
 
 --- Uncompress files from a .zip archive.
 -- @param zipfile string: pathname of .zip archive to be extracted.
 -- @return boolean: true on success, false on failure.
-function unzip(zipfile)
+function tools.unzip(zipfile)
    assert(zipfile)
-   return fs.execute(vars.SEVENZ.." x", zipfile)
+   return fs.execute_quiet(fs.Q(vars.SEVENZ).." -aoa x", zipfile)
 end
 
 --- Test is pathname is a directory.
 -- @param file string: pathname to test
 -- @return boolean: true if it is a directory, false otherwise.
-function is_dir(file)
+function tools.is_dir(file)
    assert(file)
-   return fs.execute(fs.quiet(vars.TEST.." -d " .. fs.Q(file)))
+   return fs.execute_quiet("if not exist " .. fs.Q(file.."\\").." invalidcommandname")
 end
 
 --- Test is pathname is a regular file.
 -- @param file string: pathname to test
 -- @return boolean: true if it is a regular file, false otherwise.
-function is_file(file)
+function tools.is_file(file)
    assert(file)
-   return fs.execute(vars.TEST.." -f", file)
+   return fs.execute(fs.Q(vars.TEST).." -f", file)
 end
 
 --- Download a remote file.
@@ -254,21 +246,42 @@ end
 -- resulting local filename of the remote file as the basename of the URL;
 -- if that is not correct (due to a redirection, for example), the local
 -- filename can be given explicitly as this second argument.
--- @return boolean: true on success, false on failure.
-function download(url, filename)
+-- @return (boolean, string): true and the filename on success,
+-- false and the error message on failure.
+function tools.use_downloader(url, filename, cache)
    assert(type(url) == "string")
    assert(type(filename) == "string" or not filename)
 
+   filename = fs.absolute_name(filename or dir.base_name(url))
+
+   local ok   
    if cfg.downloader == "wget" then
-      local wget_cmd = vars.WGET.." --no-check-certificate --no-cache --user-agent=\""..cfg.user_agent.." via wget\" --quiet --continue "
-      if filename then
-         return fs.execute(wget_cmd.." --output-document ", filename, url)
+      local wget_cmd = fs.Q(vars.WGET).." --no-check-certificate --no-cache --user-agent=\""..cfg.user_agent.." via wget\" --quiet "
+      if cfg.connection_timeout and cfg.connection_timeout > 0 then
+        wget_cmd = wget_cmd .. "--timeout="..tonumber(cfg.connection_timeout).." --tries=1 " 
+      end
+      if cache then
+         -- --timestamping is incompatible with --output-document,
+         -- but that's not a problem for our use cases.
+         fs.change_dir(dir.dir_name(filename))
+         ok = fs.execute_quiet(wget_cmd.." --timestamping ", url)
+         fs.pop_dir()
+      elseif filename then
+         ok = fs.execute_quiet(wget_cmd.." --output-document ", filename, url)
       else
-         return fs.execute(wget_cmd, url)
+         ok = fs.execute_quiet(wget_cmd, url)
       end
    elseif cfg.downloader == "curl" then
-      filename = filename or dir.base_name(url)
-      return fs.execute_string(vars.CURL.." -L --user-agent \""..cfg.user_agent.." via curl\" "..fs.Q(url).." 2> NUL 1> "..fs.Q(filename))
+      local curl_cmd = vars.CURL.." -f -k -L --user-agent \""..cfg.user_agent.." via curl\" "
+      if cfg.connection_timeout and cfg.connection_timeout > 0 then
+        curl_cmd = curl_cmd .. "--connect-timeout "..tonumber(cfg.connection_timeout).." " 
+      end
+      ok = fs.execute_string(curl_cmd..fs.Q(url).." 2> NUL 1> "..fs.Q(filename))
+   end
+   if ok then
+      return true, filename
+   else
+      return false
    end
 end
 
@@ -276,7 +289,7 @@ end
 -- @param archive string: Filename of archive.
 -- @return boolean : success status
 local function gunzip(archive)
-  return fs.execute(vars.SEVENZ.." x", archive)
+  return fs.execute_quiet(fs.Q(vars.SEVENZ).." -aoa x", archive)
 end
 
 --- Unpack an archive.
@@ -284,28 +297,28 @@ end
 -- filename extension.
 -- @param archive string: Filename of archive.
 -- @return boolean or (boolean, string): true on success, false and an error message on failure.
-function unpack_archive(archive)
+function tools.unpack_archive(archive)
    assert(type(archive) == "string")
 
    local ok
-   local sevenzx = vars.SEVENZ.." x"
+   local sevenzx = fs.Q(vars.SEVENZ).." -aoa x"
    if archive:match("%.tar%.gz$") then
       ok = gunzip(archive)
       if ok then
-         ok = fs.execute(sevenzx, strip_extension(archive))
+         ok = fs.execute_quiet(sevenzx, strip_extension(archive))
       end
    elseif archive:match("%.tgz$") then
       ok = gunzip(archive)
       if ok then
-         ok = fs.execute(sevenzx, strip_extension(archive)..".tar")
+         ok = fs.execute_quiet(sevenzx, strip_extension(archive)..".tar")
       end
    elseif archive:match("%.tar%.bz2$") then
-      ok = fs.execute(sevenzx, archive)
+      ok = fs.execute_quiet(sevenzx, archive)
       if ok then
-         ok = fs.execute(sevenzx, strip_extension(archive))
+         ok = fs.execute_quiet(sevenzx, strip_extension(archive))
       end
    elseif archive:match("%.zip$") then
-      ok = fs.execute(sevenzx, archive)
+      ok = fs.execute_quiet(sevenzx, archive)
    elseif archive:match("%.lua$") or archive:match("%.c$") then
       -- Ignore .lua and .c files; they don't need to be extracted.
       return true
@@ -320,28 +333,37 @@ function unpack_archive(archive)
 end
 
 local md5_cmd = {
-   md5sum = vars.MD5SUM,
-   openssl = vars.OPENSSL.." md5",
-   md5 = vars.MD5,
+   md5sum = fs.Q(vars.MD5SUM),
+   openssl = fs.Q(vars.OPENSSL).." md5",
+   md5 = fs.Q(vars.MD5),
 }
 
 --- Get the MD5 checksum for a file.
 -- @param file string: The file to be computed.
--- @return string: The MD5 checksum
-function get_md5(file)
+-- @return string: The MD5 checksum or nil + message
+function tools.get_md5(file)
    local cmd = md5_cmd[cfg.md5checker]
-   if not cmd then return nil end
-   local pipe = io.popen(cmd.." "..fs.absolute_name(file))
+   if not cmd then return nil, "no MD5 checker command configured" end
+   local pipe = io.popen(cmd.." "..fs.Q(fs.absolute_name(file)))
    local computed = pipe:read("*a")
    pipe:close()
-   if not computed then return nil end
-   return computed:match("("..("%x"):rep(32)..")")
+   if computed then
+      computed = computed:match("("..("%x"):rep(32)..")")
+   end
+   if computed then return computed end
+   return nil, "Failed to compute MD5 hash for file "..tostring(fs.absolute_name(file))
 end
 
 --- Test for existance of a file.
 -- @param file string: filename to test
 -- @return boolean: true if file exists, false otherwise.
-function exists(file)
+function tools.exists(file)
    assert(file)
-   return fs.execute(fs.quiet("if not exist " .. fs.Q(file) .. " invalidcommandname"))
+   return fs.execute_quiet("if not exist " .. fs.Q(file) .. " invalidcommandname")
 end
+
+function tools.browser(url)
+   return fs.execute(cfg.web_browser..' "Starting docs..." '..fs.Q(url))
+end
+
+return tools
