@@ -1,7 +1,9 @@
 
 --- Module implementing the LuaRocks "unpack" command.
 -- Unpack the contents of a rock.
-module("luarocks.unpack", package.seeall)
+--module("luarocks.unpack", package.seeall)
+local unpack = {}
+package.loaded["luarocks.unpack"] = unpack
 
 local fetch = require("luarocks.fetch")
 local fs = require("luarocks.fs")
@@ -9,12 +11,14 @@ local util = require("luarocks.util")
 local build = require("luarocks.build")
 local dir = require("luarocks.dir")
 
-help_summary = "Unpack the contents of a rock."
-help_arguments = "{<rock>|<name> [<version>]}"
-help = [[
+unpack.help_summary = "Unpack the contents of a rock."
+unpack.help_arguments = "[--force] {<rock>|<name> [<version>]}"
+unpack.help = [[
 Unpacks the contents of a rock in a newly created directory.
 Argument may be a rock file, or the name of a rock in a rocks server.
 In the latter case, the app version may be given as a second argument.
+
+--force   Unpack files even if the output directory already exists.
 ]]
 
 --- Load a rockspec file to the given directory, fetches the source
@@ -26,19 +30,22 @@ In the latter case, the app version may be given as a second argument.
 local function unpack_rockspec(rockspec_file, dir_name)
    assert(type(rockspec_file) == "string")
    assert(type(dir_name) == "string")
-   
+
    local rockspec, err = fetch.load_rockspec(rockspec_file)
    if not rockspec then
       return nil, "Failed loading rockspec "..rockspec_file..": "..err
    end
-   fs.change_dir(dir_name)
+   local ok, err = fs.change_dir(dir_name)
+   if not ok then return nil, err end
    local ok, sources_dir = fetch.fetch_sources(rockspec, true, ".")
    if not ok then
       return nil, sources_dir
    end
-   fs.change_dir(dir_name)
+   ok, err = fs.change_dir(sources_dir)
+   if not ok then return nil, err end
    build.apply_patches(rockspec)
    fs.pop_dir()
+   fs.pop_dir()
    return rockspec
 end
 
@@ -52,12 +59,13 @@ end
 local function unpack_rock(rock_file, dir_name, kind)
    assert(type(rock_file) == "string")
    assert(type(dir_name) == "string")
-   
+
    local ok, err, errcode = fetch.fetch_and_unpack_rock(rock_file, dir_name)
    if not ok then
       return nil, "Failed unzipping rock "..rock_file, errcode
    end
-   fs.change_dir(dir_name)
+   ok, err = fs.change_dir(dir_name)
+   if not ok then return nil, err end
    local rockspec_file = dir_name..".rockspec"
    local rockspec, err = fetch.load_rockspec(rockspec_file)
    if not rockspec then
@@ -69,7 +77,8 @@ local function unpack_rock(rock_file, dir_name, kind)
          if not ok then
             return nil, err
          end
-         fs.change_dir(rockspec.source.dir)
+         ok, err = fs.change_dir(rockspec.source.dir)
+         if not ok then return nil, err end
          build.apply_patches(rockspec)
          fs.pop_dir()
       end
@@ -83,7 +92,7 @@ end
 -- @param file string: A rockspec or .rock URL.
 -- @return boolean or (nil, string): true if successful or nil followed
 -- by an error message.
-local function run_unpacker(file)
+local function run_unpacker(file, force)
    assert(type(file) == "string")
    
    local base_name = dir.base_name(file)
@@ -95,11 +104,15 @@ local function run_unpacker(file)
    if not extension then
       return nil, file.." does not seem to be a valid filename."
    end
-   
-   if (fs.exists(dir_name)) then
+
+   local exists = fs.exists(dir_name)
+   if exists and not force then
       return nil, "Directory "..dir_name.." already exists."
    end
-   fs.make_dir(dir_name)
+   if not exists then
+      local ok, err = fs.make_dir(dir_name)
+      if not ok then return nil, err end
+   end
    local rollback = util.schedule_function(fs.delete, fs.absolute_name(dir_name))
 
    local rockspec, err
@@ -134,18 +147,20 @@ end
 -- version may also be passed.
 -- @return boolean or (nil, string): true if successful or nil followed
 -- by an error message.
-function run(...)
+function unpack.run(...)
    local flags, name, version = util.parse_flags(...)
 
    assert(type(version) == "string" or not version)
    if type(name) ~= "string" then
-      return nil, "Argument missing, see help."
+      return nil, "Argument missing. "..util.see_help("unpack")
    end
 
    if name:match(".*%.rock") or name:match(".*%.rockspec") then
-      return run_unpacker(name)
+      return run_unpacker(name, flags["force"])
    else
       local search = require("luarocks.search")
       return search.act_on_src_or_rockspec(run_unpacker, name, version)
    end
 end
+
+return unpack
